/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
/**
* DATE            CONTRIBUTOR/DEVELOPER    					NOTE
* 24-09-2008      Butano Daniela/Laboratorio di Ricerca e Sviluppo Engineering di Palermo
* 															Implementazione iniziale verso un ebXML registry
* 										    
**/
package org.spagic.registry.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.DeclarativeQueryManager;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.JAXRException;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.Query;
import javax.xml.registry.RegistryService;
import javax.xml.registry.infomodel.Classification;
import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;

import org.freebxml.omar.client.xml.registry.BusinessLifeCycleManagerImpl;
import org.freebxml.omar.client.xml.registry.DeclarativeQueryManagerImpl;
import org.freebxml.omar.client.xml.registry.infomodel.ClassificationSchemeImpl;
import org.freebxml.omar.client.xml.registry.util.JAXRUtility;
import org.freebxml.omar.client.xml.registry.util.SecurityUtil;


public class EBXMLServiceRegistryImpl extends JAXRServiceRegistry{

	protected boolean useAuthentication=true;
	protected Connection connection;
	public BusinessLifeCycleManagerImpl lcm;
    public DeclarativeQueryManagerImpl dqm;
	public BusinessQueryManager bqm; 
	
	
	public EBXMLServiceRegistryImpl() throws  ServiceRegistryException {
		Properties properties = new Properties();
		ResourceBundle bundle = ResourceBundle.getBundle("jaxr-ebxml");
		properties.setProperty("javax.xml.registry.queryManagerURL", bundle.getString("jaxr-ebxml.soap.url"));
		try {
			this.properties=properties;
			this.connectionFactory = JAXRUtility.getConnectionFactory();
		    this.connectionFactory.setProperties(properties);
		} catch (JAXRException jxre) {
			throw new ServiceRegistryException(jxre);    	
		} catch (Throwable te) {
			te.printStackTrace();
		}
	}
	
	public EBXMLServiceRegistryImpl(Properties prop) throws  ServiceRegistryException {
		super(prop);
	}
	
	public ConnectionFactory getConnectionFactory() throws ServiceRegistryException{
		try {
			return JAXRUtility.getConnectionFactory();
		}catch(JAXRException je){
			throw new ServiceRegistryException(je);
		}
	}
	
	// Connection Handling
	protected Connection getJAXRConnection() throws  ServiceRegistryException {
		Connection connection = null;
        try {
        	connection = connectionFactory.createConnection();
        	String registryUser = properties.getProperty("registry.username");
        	String registryPassword = properties.getProperty("registry.password");
  	        //configure the connection instance
  	        connection.setSynchronous(true);
            //if no authentication is used, you will log in as the RegistryGuest user
	  	    if (registryUser != null && registryPassword != null){
	  	        SecurityUtil securityUtil = SecurityUtil.getInstance();
		  	    HashSet credentials = new HashSet();
		  	    credentials.add(securityUtil.aliasToX500PrivateCredential(registryUser, registryPassword));
		  	    connection.setCredentials(credentials);
	  	    }
	  	    RegistryService service = connection.getRegistryService();
            bqm = service.getBusinessQueryManager();
            dqm = (org.freebxml.omar.client.xml.registry.DeclarativeQueryManagerImpl)service.getDeclarativeQueryManager();
            lcm = (org.freebxml.omar.client.xml.registry.BusinessLifeCycleManagerImpl)service.getBusinessLifeCycleManager();
            return connection;
        } catch(Exception e){
        	throw new ServiceRegistryException(e);
        } catch(Throwable te){
        	te.printStackTrace();
        } finally {
        	return connection;
        }
	}

	public void addValueToClassificationScheme(
			ClassificationScheme classificationScheme,
			String classificationValue) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			//se non esiste classificationValue lo aggiungo
			Collection concepts = classificationScheme.getChildrenConcepts();
			Iterator conceptIt = concepts.iterator();
			Concept conceptInCollection = null;
			while (conceptIt.hasNext()){
				conceptInCollection = (Concept)conceptIt.next();
				if(conceptInCollection.getName().getValue().equalsIgnoreCase(classificationValue))
					return;
			}
			Concept concept = blcm.createConcept(classificationScheme, blcm.createInternationalString(Locale.US, classificationValue), classificationValue);
			Collection collection = new ArrayList();
			collection.add(concept);
			blcm.saveConcepts(collection);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
		
	}

	public void classifyServiceByTaxonomy(Service service,
			ClassificationScheme classificationScheme,
			String classificationValue) throws ServiceRegistryException {
			Connection registryConnection = null;
			try {
				registryConnection = getJAXRConnection();
				//Create services and service
				RegistryService registryService = registryConnection.getRegistryService();
				BusinessLifeCycleManager lcm=registryService.getBusinessLifeCycleManager();
				DeclarativeQueryManager  dqm=registryService.getDeclarativeQueryManager();
				String query="select c.* from Concept c where c.code='"+classificationValue+"'";
				Query q=dqm.createQuery(Query.QUERY_TYPE_SQL,query);
				BulkResponse resp=dqm.executeQuery(q);
				Collection<Concept> cpts = (Collection<Concept>)resp.getCollection();
				Concept cpt=null;
				
				if (cpts.size() > 0) {
					for (int index=0; index<cpts.size(); index++){
						cpt = (Concept) cpts.toArray()[index];
						if (cpt.getClassificationScheme().getKey().equals(classificationScheme.getKey()))
							break;
					}
				} else {
					throw new ServiceRegistryException("No concept ["+classificationValue+"] found ");
				}
				Classification cls=lcm.createClassification(cpt);
				System.out.println("Added Classification "+cls);
				service.addClassification(cls);
				ArrayList<Service> services =new ArrayList<Service>();
				services.add(service);
				lcm.saveServices(services);
	            				
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public void deleteClassificationScheme(
			ClassificationScheme classificationScheme)
			throws ServiceRegistryException {
	}

	public void deleteOrganization(Organization organization)
			throws ServiceRegistryException {
		// TODO Auto-generated method stub
		
	}

	public void deleteService(Organization organization, Service service)
			throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			Organization org = (Organization)bqm.getRegistryObject(organization.getKey().getId(), LifeCycleManager.ORGANIZATION);
			Service ser = (Service)bqm.getRegistryObject(service.getKey().getId(), LifeCycleManager.SERVICE);
			ser.setProvidingOrganization(org);
			org.removeService(ser);
			publishOrganization(org);
			
//			removeAllClassificationForService(ser);
//			
//			//remove servicebinding
//			Collection serviceBindings = service.getServiceBindings();
//			Iterator itServiceBindings = serviceBindings.iterator();
//			while (itServiceBindings.hasNext())
//				service.removeServiceBinding((ServiceBinding)itServiceBindings.next());
			
			Key key = service.getKey();
			Collection keys = new ArrayList();
			keys.add(key);
			BulkResponse response = blcm.deleteObjects(keys);
			
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
		
		
		
	}
	public ClassificationScheme newClassificationScheme(String cSchemeName, String cSchemeDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			BusinessLifeCycleManagerImpl lcm = (BusinessLifeCycleManagerImpl)registryService.getBusinessLifeCycleManager();
			ClassificationScheme cScheme = new ClassificationSchemeImpl(lcm);
			cScheme.setName(blcm.createInternationalString(Locale.US, cSchemeName));
			cScheme.setDescription(blcm.createInternationalString(Locale.US, cSchemeDescription));
			return cScheme;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}	
	public List<ClassificationScheme> getAllClassificationScheme()
			throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			
			List<ClassificationScheme> returnedCSchemes = new ArrayList<ClassificationScheme>();
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BulkResponse response= bqm.findClassificationSchemes(null, null, null, null);
			Collection<ClassificationScheme> schemes = (Collection<ClassificationScheme>)response.getCollection();
			System.out.println("---#"+schemes.size()+" schemes founded");
			List<Organization> result = new ArrayList<Organization>();
			for (ClassificationScheme org : schemes){
				returnedCSchemes.add(org);
			}
			return returnedCSchemes;
		} catch (Exception e) {
			e.printStackTrace();
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
		
		
	}

	public List<String> getAllClassificationsForService(Organization org,
			Service ser) throws ServiceRegistryException {
		try {
			Collection<Classification> classifications = ser.getClassifications();
			List classificationForService = new ArrayList();
			for(Classification classification : classifications) 
				classificationForService.add(classification.getClassificationScheme().getName().getValue()+ ";" + classification.getName().getValue());
			return classificationForService;
		}catch (JAXRException e){
			throw new ServiceRegistryException(e);
		} 
	}
	
	public List<Service> getServicesClassifiedByTaxonomy(String classificationSchemeName, String classificationValue) throws ServiceRegistryException {
		ClassificationScheme cScheme = getClassificationSchemeByName(classificationSchemeName);
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			
			DeclarativeQueryManager  dqm=registryService.getDeclarativeQueryManager();
			String query="select c.* from Concept c where c.code='"+classificationValue+"'";
			Query q=dqm.createQuery(Query.QUERY_TYPE_SQL,query);
			BulkResponse resp=dqm.executeQuery(q);
			Collection<Concept> cpts = (Collection<Concept>)resp.getCollection();
			Concept cpt=null;
			
			if (cpts.size() > 0) {
				for (int index=0; index<cpts.size(); index++){
					cpt = (Concept) cpts.toArray()[index];
					if (cpt.getClassificationScheme().getName().getValue().equals(classificationSchemeName))
						break;
				}
			} else {
				throw new ServiceRegistryException("No concept ["+classificationValue+"] found ");
			}
			Classification cls=blcm.createClassification(cpt);
			Collection classifications = new ArrayList();
			classifications.add(cls);
	    	
			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_ASC);
			BulkResponse response = bqm.findServices(null, findQualifiers, null, classifications,null);
			Collection services = response.getCollection();
			List<Service> servicesList = new ArrayList<Service>();
			Iterator svcIter = services.iterator();
			while (svcIter.hasNext()) {
				Service svc = (Service) svcIter.next();
				servicesList.add(svc);
			}
			return servicesList;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public ClassificationScheme getClassificationSchemeByName(String schemeName)
			throws ServiceRegistryException {

		Connection registryConnection = null;
		try {
			List<ClassificationScheme> returnedCSchemes = new ArrayList<ClassificationScheme>();
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(schemeName);
			
			BulkResponse response= bqm.findClassificationSchemes(null, namePatterns, null, null);
			
			Collection<ClassificationScheme> schemes = (Collection<ClassificationScheme>)response.getCollection();
			if (schemes.size() > 0) {
				
				ClassificationScheme scheme = (ClassificationScheme) schemes.toArray()[0];
				return scheme;
			} else {
				return null;
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public List<String> getClassificationSchemeValues(
			String classificationSchemeName) throws ServiceRegistryException {
       try {    
    	   ClassificationScheme scheme_=this.getClassificationSchemeByName(classificationSchemeName);
    	   Collection<Concept> concepts=scheme_.getChildrenConcepts();
		   List<String> returns = new ArrayList<String>();
		   String str = null;
		   for (Concept m : concepts){
				str = new String();
				str += m.getValue();
				returns.add(str);
			}
			return returns;
		} catch (Exception e) {
			//e.printStackTrace();
			throw new ServiceRegistryException(e);
		} finally {
			
		}
	}

	public Service publishService(Organization organization, Service service,
			List<ServiceBinding> bindings) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			//Create services and service
			int binnum=0;
			if (bindings!=null)
				binnum=bindings.size();
			System.out.println("Publishing service "+service.getName().getValue()+" bindings"+binnum);
			
			Collection<Service> services = new ArrayList<Service>();
			//	 Add service bindings to service
			if (bindings != null)
				service.addServiceBindings(bindings);
			//   Add service to services, then add services to organization
			service.setProvidingOrganization(organization);

			services.add(service);
			organization.addServices(services);
			publishOrganization(organization);
			return service;
	} catch (Exception e) {
		throw new ServiceRegistryException(e);
	}finally {
		closeJAXRConnection(registryConnection);
	}
	}

	public void removeAllValuesFromClassificationScheme(
			ClassificationScheme classificationScheme,
			String classificationValue) throws ServiceRegistryException {
		// TODO Auto-generated method stub
		
	}
	
	public void removeAllClassificationForServiceByClassification(ClassificationScheme cScheme, Service service) throws ServiceRegistryException {
		// TODO Auto-generated method stub
	}

	public void removeValueFromClassificationScheme(
			ClassificationScheme classificationScheme,
			String classificationValue) throws ServiceRegistryException {
		// TODO Auto-generated method stub
		
	}

	public ServiceBinding setWsdlLocation(String serviceName,
			ServiceBinding binding, String wsdlLocation)
			throws ServiceRegistryException {
		// TODO Auto-generated method stub
		return null;
	}

}
