/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.registry.utils;

import java.net.PasswordAuthentication;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.Set;

import javax.sql.DataSource;
import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.JAXRException;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.RegistryService;
import javax.xml.registry.infomodel.Classification;
import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.ExternalLink;
import javax.xml.registry.infomodel.InternationalString;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;
import javax.xml.registry.infomodel.SpecificationLink;

import org.freebxml.omar.client.xml.registry.util.JAXRUtility;


public abstract class JAXRServiceRegistry implements IServiceRegistry {
	
	protected Properties properties = null;
	protected ConnectionFactory connectionFactory = null;
	protected DataSource uddiDataSource = null;
	protected Locale systemDefaultLocale = null;
	
	public JAXRServiceRegistry() throws  ServiceRegistryException {
	}
	
	public JAXRServiceRegistry(Properties prop) throws  ServiceRegistryException {
		try {
			this.properties = prop;
	        this.connectionFactory = JAXRUtility.getConnectionFactory();
	        this.connectionFactory.setProperties(properties);
		}catch (JAXRException jxre) {
			System.out.println(jxre);
			throw new ServiceRegistryException(jxre);    	
	    }
	}
	protected abstract ConnectionFactory getConnectionFactory()throws ServiceRegistryException;
	
		// Connection Handling
	protected Connection getJAXRConnection() throws  ServiceRegistryException {
        try{
        	Connection connection = connectionFactory.createConnection();
        	String registryUser = properties.getProperty("registry.username");
        	String registryPassword = properties.getProperty("registry.password");
        	PasswordAuthentication pa = new PasswordAuthentication(registryUser, registryPassword.toCharArray());
        	Set credentials = new HashSet();
        	credentials.add(pa);
        	connection.setCredentials(credentials);
        	return connection;
        }catch(Exception e){
        	throw new ServiceRegistryException(e);
        }
	}
	
	protected static void closeJAXRConnection(Connection aJAXRConnection)  {
		try{
			if (aJAXRConnection !=null && !aJAXRConnection.isClosed())
				aJAXRConnection.close();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}


	public Organization[] getOrganizationsByName(String organizationName)
	throws ServiceRegistryException {
Connection registryConnection = null;

try {
	registryConnection = getJAXRConnection();
	RegistryService registryService = registryConnection
			.getRegistryService();
	BusinessQueryManager bqm = registryService
			.getBusinessQueryManager();

	Collection<String> findQualifiers = new ArrayList<String>();
	findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
	Collection<String> namePatterns = new ArrayList<String>();
	namePatterns.add(organizationName);

	BulkResponse response = bqm.findOrganizations(findQualifiers,
			namePatterns, null, null, null, null);
	Collection orgs = response.getCollection();
	System.out.println(" Orgs.size["+orgs.size()+"]");
	Organization[] ret = new Organization[orgs.size()];
	int i=0;
	for (Iterator it = orgs.iterator(); it.hasNext();){
		ret[i] = (Organization)it.next();
		i++;
	}
	return ret;
} catch (Exception e) {
	//e.printStackTrace();
	throw new ServiceRegistryException(e);
} finally {
	closeJAXRConnection(registryConnection);
}
}
	

	public Organization getOrganizationByName(String organizationName)
			throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection
					.getRegistryService();
			BusinessQueryManager bqm = registryService
					.getBusinessQueryManager();
			
			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(organizationName);

			BulkResponse response = bqm.findOrganizations(findQualifiers,
					namePatterns, null, null, null, null);
			Collection orgs = response.getCollection();
			System.out.println(" Orgs.size["+orgs.size()+"]");
			if (orgs.size() > 1) {
				throw new ServiceRegistryException(
						"Found more than one result for this organization name");
			} else if (orgs.size() == 1) {
				Organization org = (Organization) orgs.toArray()[0];
				return org;
			} else {
				return null;
			}
			
		} catch (Exception e) {
			//e.printStackTrace();
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
	}


	public abstract void deleteOrganization(Organization organization) throws ServiceRegistryException;

	public abstract void deleteService(Organization organization, Service service) throws ServiceRegistryException;
	
	public abstract List<Service> getServicesClassifiedByTaxonomy(String classificationSchemeName, String classificationValue) throws ServiceRegistryException;


	public List<Service> getServicesForOrganizations(Organization organization) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			Organization org = (Organization)bqm.getRegistryObject(organization.getKey().getId(), LifeCycleManager.ORGANIZATION);
			
			List<Service> servicesList = new ArrayList<Service>();
		
			Collection services = org.getServices();
			Iterator svcIter = services.iterator();
			while (svcIter.hasNext()) {
				Service svc = (Service) svcIter.next();
				servicesList.add(svc);
			}
		return servicesList;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public Organization publishOrganization(Organization organization) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
        
			Collection orgs = new ArrayList();
			orgs.add(organization);

			//Save the Organization in the UDDI registry.
			BulkResponse bulkResponse = blcm.saveOrganizations(orgs);

			//Obtain the Organization's Key (the UDDI businessEntity's businessKey) from the response.
			if (bulkResponse.getExceptions() == null)
			{
				//1 Organization was saved, so 1 key will be returned in the response collection
				Collection responses = bulkResponse.getCollection();
				Key organizationKey = (Key)responses.iterator().next();
				organization.setKey(organizationKey);
				return organization;
			}else{
				return null;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	

	public Service publishService(Organization organization, Service service, List<ServiceBinding> serviceBindings) throws ServiceRegistryException {
		
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			//Create services and service
			Collection<Service> services = new ArrayList<Service>();
			//	 Add service bindings to service
			if (serviceBindings != null)
				service.addServiceBindings(serviceBindings);
			//   Add service to services, then add services to organization
			service.setProvidingOrganization(organization);

			services.add(service);
			organization.addServices(services);
			publishOrganization(organization);
			return service;
	} catch (Exception e) {
		throw new ServiceRegistryException(e);
	}finally {
		closeJAXRConnection(registryConnection);
	}
	}

	public ServiceBinding setWsdlLocation(String serviceName, ServiceBinding binding, String wsdlLocation) throws ServiceRegistryException{
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
		
			if (wsdlLocation != null){
				Concept specConcept =
					blcm.createConcept(null, blcm.createInternationalString("Concept for service "+ serviceName), "");
				InternationalString s = blcm.createInternationalString("Concept for service "+ serviceName);
				specConcept.setDescription(s);
				ExternalLink wsdlLink =
					blcm.createExternalLink(wsdlLocation,serviceName + "WSDL");
				specConcept.addExternalLink(wsdlLink);
    		
				// Classifying the concept as WSDL
				// String uuid_types = "uuid:c1acf26d-9672-4404-9d70-39b756e62ab4";
				String schemeName = "uddi-org:types";
				ClassificationScheme uddiOrgTypes =
					(ClassificationScheme) bqm.findClassificationSchemeByName(null, schemeName);
    			
				Classification wsdlSpecClassification =
					blcm.createClassification(uddiOrgTypes,
							"wsdlSpec", "wsdlSpec");
				specConcept.addClassification(wsdlSpecClassification);
    		
    		
				Collection<Concept> concepts = new ArrayList<Concept>();
				concepts.add(specConcept);
    			BulkResponse concResponse = blcm.saveConcepts(concepts);
    		
    			String conceptKeyId = null;
    			Collection concExceptions = concResponse.getExceptions();
    			Key concKey = null;
    			if (concExceptions == null) {
    				System.out.println("WSDL Specification Concept saved");
    				Collection keys = concResponse.getCollection();
    				Iterator<Key> keyIter = keys.iterator();
    				if (keyIter.hasNext()) {
    					concKey = (Key) keyIter.next();
    					conceptKeyId = concKey.getId();
    					System.out.println("Concept key is " + conceptKeyId);
    				}
    			}
    		
    		
    			specConcept = (Concept) bqm.getRegistryObject(conceptKeyId,
    					LifeCycleManager.CONCEPT);
    			
    			SpecificationLink specLink = blcm.createSpecificationLink();
    			specLink.setSpecificationObject(specConcept);
    			binding.addSpecificationLink(specLink);
			}
			return binding;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public Organization newOrganization(String organizationName, String organizationDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
	        BusinessLifeCycleManager lifeCycleManager = registryService.getBusinessLifeCycleManager();
	    	InternationalString iOrganizatioName = lifeCycleManager.createInternationalString(Locale.US, organizationName);
	        Organization organization = lifeCycleManager.createOrganization(iOrganizatioName);
	        InternationalString iOrganizationDescription = lifeCycleManager.createInternationalString(Locale.US, organizationDescription);
	        organization.setDescription(iOrganizationDescription);
	        return organization;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public Service newService(String serviceName, String serviceDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			InternationalString iServiceName = blcm.createInternationalString(Locale.US, serviceName);
			Service service = blcm.createService(iServiceName);
			InternationalString iServiceDescription = blcm.createInternationalString(Locale.US,serviceDescription);
			service.setDescription(iServiceDescription);
			return service;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public ServiceBinding newServiceBinding(String serviceName, String serviceBindingLocation) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();	
			ServiceBinding binding = blcm.createServiceBinding();
			InternationalString iServiceBindingDescritpion = blcm.createInternationalString(Locale.US, "Binding For " + serviceName );
			binding.setDescription(iServiceBindingDescritpion);
			//	 allow us to publish a fictitious URI without an error
			binding.setValidateURI(false);
			binding.setAccessURI(serviceBindingLocation);
			return binding;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}


	public abstract void addValueToClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException;

	public abstract void deleteClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException;

	public abstract ClassificationScheme newClassificationScheme(String cSchemeName, String cSchemeDescription) throws ServiceRegistryException;


	public ClassificationScheme publishClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
        
			Collection classifications = new ArrayList();
			classifications.add(classificationScheme);

			//Save the Organization in the UDDI registry.
			BulkResponse bulkResponse = blcm.saveClassificationSchemes(classifications);

			//Obtain the Organization's Key (the UDDI businessEntity's businessKey) from the response.
			if (bulkResponse.getExceptions() == null)
			{
				//1 Organization was saved, so 1 key will be returned in the response collection
				Collection responses = bulkResponse.getCollection();
				Key classificationKey = (Key)responses.iterator().next();
				classificationScheme.setKey(classificationKey);
				return classificationScheme;
			}else{
				return null;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}		

			
	}

	public abstract ClassificationScheme getClassificationSchemeByName(String cSchemeName) throws ServiceRegistryException;

	public Service getServiceByOrganizationNameAndServiceName(String organizationName, String serviceName) throws ServiceRegistryException {
		Organization org = getOrganizationByName(organizationName);
		return getServiceByOrganizationAndServiceName(org, serviceName);
	}
	
	public Service getServiceByOrganizationAndServiceName(Organization org, String serviceName) throws ServiceRegistryException {	
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			

			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(serviceName);
              
			
			BulkResponse response = bqm.findServices(org.getKey(), null, namePatterns,null,null);
			Collection services = response.getCollection();			
			if (services.size() > 1){
				throw new ServiceRegistryException(" Founded more that one service named ["+serviceName+"] in Organization["+org.getName()+"]");
			}else{
				Service aService = (Service)services.iterator().next();
							
				return aService;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
			
	}

	public abstract void classifyServiceByTaxonomy(Service service, ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException;

	public void removeAllClassificationForService(Service service) throws ServiceRegistryException {
		try {
			service.removeClassifications(service.getClassifications());
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} 
	}

	public abstract List<ClassificationScheme> getAllClassificationScheme() throws ServiceRegistryException;
	
	public List<Organization> getAllOrganizations() throws ServiceRegistryException {
			Connection registryConnection = null;
			try {
				registryConnection = getJAXRConnection();
				RegistryService registryService = registryConnection
						.getRegistryService();
				BusinessQueryManager bqm = registryService
						.getBusinessQueryManager();

				Collection<String> findQualifiers = new ArrayList<String>();
				findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
				Collection<String> namePatterns = new ArrayList<String>();
				namePatterns.add("%");
			
				BulkResponse response = bqm.findOrganizations(findQualifiers,
						namePatterns, null, null, null, null);
				Collection<Organization> orgs = (Collection<Organization>)response.getCollection();
				List<Organization> result = new ArrayList<Organization>();
				for (Organization org : orgs){
					result.add(org);
				}
				return result;
			} catch (Exception e) {
				throw new ServiceRegistryException(e);
			} finally {
				closeJAXRConnection(registryConnection);
			}
	}

	public abstract List<String> getClassificationSchemeValues(String classificationSchemeName) throws ServiceRegistryException;
	
	public abstract List<String> getAllClassificationsForService(Organization org, Service ser) throws ServiceRegistryException;

	public void finalizeRegistry() {
		Locale.setDefault(this.systemDefaultLocale);
		
	}



	public void initRegistry() {
		this.systemDefaultLocale = Locale.getDefault();
		Locale.setDefault(Locale.US);
		
	}


	
	
	
}
